  SUBROUTINE bc(v, m, mp, k, p)
! PDE: Laplacian u = 0;      0<=x<=1;  0<=y<=1
! B.C.: u(x,0)=sin(pi*x); u(x,1)=sin(pi*x)*exp(-pi); u(0,y)=u(1,y)=0
! SOLUTION: u(x,y)=sin(pi*x)*exp(-pi*y)   
    IMPLICIT NONE
    INTEGER m, mp, k, p, j
    REAL*8, DIMENSION(0:m+1,0:mp+1) :: v
    REAL*8, DIMENSION(:,:), POINTER :: c
    REAL*8, DIMENSION(0:m+1) :: y0

    y0 = sin(3.141593*(/(j,j=0,m+1)/)/(m+1))

    IF(p .eq. 1) THEN      
      v = 0.0d0
      v(:,0) = y0
      v(:,m+1) = y0*exp(-3.141593)
    END IF
    RETURN
  END SUBROUTINE bc

  real*8 function output(a,n,f)
      implicit none
      integer n
      real*8 a(n,n), f(n,n)
      integer i, j
      real*8 resid, error

      resid = 0.D0
      error = 0.D0
!$omp parallel do default(shared) private(i,j,resid) reduction(+:error)
      do j = 2, n-1
        do i = 2, n-1
           resid= abs((a(i+1,j)+a(i-1,j)+a(i,j+1)+a(i,j-1))/4-a(i,j)-f(i,j))
           error = error + resid * resid
        end do
      end do
      error = sqrt(error)
      print *,'Residual is:', error

      output = error

      return
      end

  real*8 function relative()
      relative = 0.02
      return 
      end
  real*8 function init(a,n,f)
      implicit none
      integer n
      real*8 a(n,n),f(n,n), output,relative
      integer i,j

!$omp parallel do default(shared) private(i,j)
      do 100 j = 1,n
      do 100 i = 1,n
        a(i,j) = 0 
        f(i,j) = 0
  100 continue

      call bc(a, n-2, n-2, 0, 1)
       
      init = output(a,n,f) * relative()

      return
      end
